from uuid import UUID

from starlette import status
import fastapi
from tortoise.exceptions import ValidationError

from base_ctrl import BaseController
from exceptions import common as common_exc, http as http_exc
from .services import StatusService


class StatusController(BaseController):
    status_service = StatusService()

    async def get_list(self, **kwargs):
        return await self.status_service.get_status_list(**kwargs)

    async def get(self, id: UUID):
        try:
            return await self.status_service.get_status(id)

        except common_exc.NotFoundException as e:
            raise http_exc.HTTPNotFoundException(detail=str(e))

    async def create(self, **kwargs):
        try:
            return await self.status_service.create_status(**kwargs)

        except common_exc.CreateException as e:
            raise http_exc.HTTPBadRequestException(detail=str(e))

        except ValidationError as e:
            raise http_exc.HTTPBadRequestException(detail=f"Validation error: {str(e)}")

    async def update(self, id: UUID, **kwargs):
        try:
            return await self.status_service.update_status(id, **kwargs)

        except common_exc.UpdateException as e:
            raise http_exc.HTTPBadRequestException(detail=str(e))

    async def delete(self, id: UUID):
        try:
            await self.status_service.delete_status(id)
            return fastapi.responses.Response(status_code=status.HTTP_204_NO_CONTENT)

        except common_exc.DeleteException as e:
            raise http_exc.HTTPBadRequestException(detail=str(e))

        except common_exc.NotFoundException as e:
            raise http_exc.HTTPNotFoundException(detail=str(e))
